import curses
import logging

import screen
import userinput


class Game(object):
    def __init__(self, scr, cur_userinput, args):
        self.screen = scr
        self.userinput = cur_userinput
        self.args = args
        self._running = True

        self.tick_length_ms = 100
        self.only_draw_on_ticks = True

        self.game_objects = []

    def get_tick_length_ms(self):
        return self.tick_length_ms

    def add_object(self, o):
        self.game_objects.append(o)
        if o.figure:
            self.screen.add_figure(o.figure)

    def iterate(self, ticks):
        for o in self.game_objects:
            if o.active:
                o.iterate(ticks)

    def handle_input(self, incmd):
        pass

    def quit(self):
        pass

    def signal_quit(self):
        self._running = False


class GameError(Exception):
    pass


class GameObject(object):
    def __init__(self, active=False, *args, **kwargs):
        self.active = active
        self.figure = None
        self.game = kwargs.get('game')

    def set_figure(self, figure):
        self.figure = figure
        self.figure.set_visible(self.active)

    def set_active(self, active=True):
        self.active = active
        if self.figure:
            self.figure.set_visible(self.active)

    def iterate(self, ticks):
        pass
