#!/usr/bin/env python

import curses
import logging
import pickle
import sys

import game
import gameworld
import screen
import userinput


class FillRect(screen.Figure):
    def __init__(self, ch, color, *args, **kwargs):
        super(FillRect, self).__init__(*args, **kwargs)
        self.rect_char = ch
        self.rect_attr = curses.color_pair(color)
    def get_char_and_attr(self, x, y):
        return self.rect_char, self.rect_attr
    def __str__(self):
        return ('FillRect(ch={}, attr={}, width={}, height={}, x_pos={}, '
                'y_pos={}, z_index={}, visible={})'.format(
                    self.rect_char, self.rect_attr, self.width, self.height,
                    self.x_pos, self.y_pos, self.z_index, self.visible))


class HollowRect(screen.Figure):
    def __init__(self, ch, color, *args, **kwargs):
        super(HollowRect, self).__init__(*args, **kwargs)
        self.rect_char = ch
        self.rect_attr = curses.color_pair(color)
    def get_char_and_attr(self, x, y):
        if (x == 0 or x == self.width - 1 or
            y == 0 or y == self.height - 1):
            return self.rect_char, self.rect_attr
        else:
            return (None, None)
    def __str__(self):
        return ('HollowRect(ch={}, attr={}, width={}, height={}, x_pos={}, '
                'y_pos={}, z_index={}, visible={})'.format(
                    self.rect_char, self.rect_attr, self.width, self.height,
                    self.x_pos, self.y_pos, self.z_index, self.visible))


class RectGame(game.Game):
    def __init__(self, *args, **kwargs):
        super(RectGame, self).__init__(*args, **kwargs)

        self.fig_green_rect = FillRect(ord('a'), curses.COLOR_GREEN, 4, 4, 2, 2, 1)
        self.screen.add_figure(self.fig_green_rect)
        self.fig_red_rect = FillRect(ord('b'), curses.COLOR_RED, 4, 4, 4, 4, 2)
        self.screen.add_figure(self.fig_red_rect)
        self.fig_blue_rect = FillRect(ord('c'), curses.COLOR_BLUE, 4, 4, 6, 6, 3)
        self.screen.add_figure(self.fig_blue_rect)

    def iterate(self, ticks):
        pass

    def handle_input(self, cmd):
        if (cmd == userinput.LEFT and
            self.fig_red_rect.x_pos > 0):
            self.fig_red_rect.x_pos -= 1
        elif (cmd == userinput.RIGHT and
              self.fig_red_rect.x_pos < self.screen.stdscr.getmaxyx()[1] - self.fig_red_rect.width):
            self.fig_red_rect.x_pos += 1
        elif (cmd == userinput.UP and
              self.fig_red_rect.y_pos > 0):
            self.fig_red_rect.y_pos -= 1
        elif (cmd == userinput.DOWN and
              self.fig_red_rect.y_pos < self.screen.stdscr.getmaxyx()[0] - self.fig_red_rect.height):
            self.fig_red_rect.y_pos += 1


class HollowRectGame(game.Game):
    def __init__(self, *args, **kwargs):
        super(HollowRectGame, self).__init__(*args, **kwargs)

        self.fig_green_rect = HollowRect(ord('a'), curses.COLOR_GREEN, 4, 4, 2, 2, 1)
        self.screen.add_figure(self.fig_green_rect)
        self.fig_red_rect = HollowRect(ord('b'), curses.COLOR_RED, 4, 4, 4, 4, 2)
        self.screen.add_figure(self.fig_red_rect)
        self.fig_blue_rect = HollowRect(ord('c'), curses.COLOR_BLUE, 4, 4, 6, 6, 3)
        self.screen.add_figure(self.fig_blue_rect)

    def iterate(self, ticks):
        pass

    def handle_input(self, cmd):
        if (cmd == userinput.LEFT and
            self.fig_red_rect.x_pos > 0):
            self.fig_red_rect.x_pos -= 1
        elif (cmd == userinput.RIGHT and
              self.fig_red_rect.x_pos < self.screen.stdscr.getmaxyx()[1] - self.fig_red_rect.width):
            self.fig_red_rect.x_pos += 1
        elif (cmd == userinput.UP and
              self.fig_red_rect.y_pos > 0):
            self.fig_red_rect.y_pos -= 1
        elif (cmd == userinput.DOWN and
              self.fig_red_rect.y_pos < self.screen.stdscr.getmaxyx()[0] - self.fig_red_rect.height):
            self.fig_red_rect.y_pos += 1


class RectSnakeGame(game.Game):
    class SnakePart(object):
        WIDTH = 4
        HEIGHT = 4

        def __init__(self, color, x_pos, y_pos, z_index, scr_width, scr_height):
            self.fig = FillRect(ord('#'), color, self.WIDTH, self.HEIGHT, x_pos, y_pos, z_index)
            self.horiz_vel = 1
            self.vert_vel = 1
            self.scr_width = scr_width
            self.scr_height = scr_height

        def move(self):
            if ((self.fig.x_pos + self.fig.width + self.horiz_vel > self.scr_width) or
                (self.fig.x_pos + self.horiz_vel < 0)):
                self.horiz_vel *= -1
            if ((self.fig.y_pos + self.fig.height + self.vert_vel > self.scr_height) or
                (self.fig.y_pos + self.vert_vel < 0)):
                self.vert_vel *= -1
            self.fig.x_pos += self.horiz_vel
            self.fig.y_pos += self.vert_vel

    def __init__(self, *args, **kwargs):
        super(RectSnakeGame, self).__init__(*args, **kwargs)

        self.tick_length_ms = 50

        colors = [curses.COLOR_BLUE,
                  curses.COLOR_CYAN,
                  curses.COLOR_GREEN,
                  curses.COLOR_MAGENTA,
                  curses.COLOR_RED,
                  curses.COLOR_WHITE,
                  curses.COLOR_YELLOW]
        self.parts = []
        for i, color in enumerate(colors):
            part = RectSnakeGame.SnakePart(color, i, i, i,
                                           self.screen.width, self.screen.height)
            self.screen.add_figure(part.fig)
            self.parts.append(part)

    def iterate(self, ticks):
        for part in self.parts:
            for x in xrange(ticks):
                part.move()

    def handle_input(self, cmd):
        pass


if __name__ == '__main__':
    #world = gameworld.GameWorld(RectGame)
    #world = gameworld.GameWorld(HollowRectGame)
    world = gameworld.GameWorld(RectSnakeGame)
    sys.exit(world.main(sys.argv[1:]))
